PROGRAM FISHCARD (input, output, dfile, rfile);
(* Flash card review software *)
(* by David S. Mueller - dsm42@gammaquad.com *)
(* Copyright 1996-2000 David S. Mueller *)

CONST
  VERSION = '1.2';
  ENDFILE = 28;
  ENDRECORD = 30;
  ENDUNIT = 31;
  TEMPFILE = 'FISHCARD.TMP';
  MAXFILE = 12;
  MAXCARD = 1000;
  BLANK = ' ';

TYPE
  fnametype = packed array [1..MAXFILE] of char;
  carddatatype = packed array [1..MAXCARD] of char;

VAR
  choice : char;

(* ------------------------------------------------------------------------- *)

PROCEDURE DrawCards;
(* Draws the card outline *)

BEGIN
  clrscr;
  textcolor (green);
  writeln ('ͻ');
  writeln ('                                                                            ');
  writeln ('                                                                            ');
  writeln ('                                                                            ');
  writeln ('                                                                            ');
  writeln ('                                                                            ');
  writeln ('                                                                            ');
  writeln ('                                                                            ');
  writeln ('                                                                            ');
  writeln ('ĺ');
  writeln ('                                                                            ');
  writeln ('                                                                            ');
  writeln ('                                                                            ');
  writeln ('                                                                            ');
  writeln ('                                                                            ');
  writeln ('                                                                            ');
  writeln ('                                                                            ');
  writeln ('                                                                            ');
  writeln ('ͼ')
END; (*DrawCards*)

(* ------------------------------------------------------------------------- *)

PROCEDURE DrawQAText;
(* Writes Question & Answer on the card *)

BEGIN
  textcolor (lightred);
  gotoxy (35,1);
  write (' QUESTION ');
  gotoxy (36,10);
  write (' ANSWER ')
END; (*DrawQAText*)

(* ------------------------------------------------------------------------- *)

PROCEDURE DisplayMenu;
(* Displays the menu *)

BEGIN
  clrscr;
  textcolor (magenta);
  writeln ('FishCard version ',VERSION);
  writeln ('Copyright 1996-2000 by David S. Mueller');
  textcolor (15);
  writeln;
  writeln ('1. Create New FishCards');
  writeln ('2. Review FishCards');
  writeln ('3. Import/Export - Text <--> FishCard');
  writeln ('4. Quit');
  writeln;
  write ('Please select an option from the menu above: ')
END; (*DisplayMenu*)

(* ------------------------------------------------------------------------- *)

PROCEDURE GetUserChoice (var choice : char);
(* Gets the user's choice *)

BEGIN
  textcolor (cyan);
  read (kbd, choice);
  while not (choice in ['1', '2', '3', '4']) do
    read (kbd, choice);
  writeln (choice)
END; (*GetUserChoice*)

(* ------------------------------------------------------------------------- *)

PROCEDURE ShowMenu (var choice : char);
(* Handles menu disply *)

BEGIN
  DisplayMenu;
  GetUserChoice (choice)
END; (*ShowMenu*)

(* ------------------------------------------------------------------------- *)

PROCEDURE InitString (var STR : fnametype);
(* Fills STR with blanks *)

VAR
  INDEX : INTEGER;

BEGIN
  FOR INDEX := 1 TO MAXFILE DO
    STR[INDEX] := BLANK
END; (*InitString*)

(* ------------------------------------------------------------------------- *)

PROCEDURE InitCardData (var STR : carddatatype);
(* Fills STR with blanks *)

VAR
  INDEX : INTEGER;

BEGIN
  FOR INDEX := 1 TO MAXCARD DO
    STR[INDEX] := BLANK
END; (*InitCardData*)

(* ------------------------------------------------------------------------- *)

FUNCTION Length (var STR : carddatatype) : INTEGER;
(* Finds the length of STR *)

VAR
  POS : INTEGER;

BEGIN
  POS := MAXCARD;
  WHILE (POS > 1) AND (STR[POS] = BLANK) DO
    POS := POS - 1;
  IF STR[POS] = BLANK
    THEN LENGTH := 0
    ELSE LENGTH := POS
END; (*Length*)

(* ------------------------------------------------------------------------- *)

PROCEDURE GetString (var WORD : fnametype);
(* Reads a string from the user *)

VAR
  POS : INTEGER;
  CH : CHAR;

BEGIN
  InitString (WORD);
  POS := 1;
  READ (kbd, CH);
  WHILE (ORD(CH) <> 13) AND (POS < MAXFILE + 1) DO
    BEGIN
      IF (ORD(CH) = 127) OR (ORD(CH) = 8)
        THEN
          BEGIN
            IF POS > 1
              THEN
                BEGIN
                  POS := POS - 1;
                  WRITE (CH)
                END
          END
        ELSE
          BEGIN
            WRITE (CH);
            WORD[POS] := CH;
            POS := POS + 1
          END;
      READ (kbd, CH)
    END
END; (*GetString*)

(* ------------------------------------------------------------------------- *)

PROCEDURE GetCardData (var WORD : carddatatype);
(* Reads a string from the user *)

VAR
  POS : INTEGER;
  CH : CHAR;

BEGIN
  InitCardData (WORD);
  POS := 1;
  READ (kbd, CH);
  WHILE (ORD(CH) <> 13) AND (POS < MAXCARD + 1) DO
    BEGIN
      IF (ORD(CH) = 127) OR (ORD(CH) = 8)
        THEN
          BEGIN
            IF POS > 1
              THEN
                BEGIN
                  POS := POS - 1;
                  WRITE (CH)
                END
          END
        ELSE
          BEGIN
            WRITE (CH);
            WORD[POS] := CH;
            POS := POS + 1
          END;
      READ (kbd, CH)
    END
END; (*GetCardData*)

(* ------------------------------------------------------------------------- *)

PROCEDURE ShowText (var dfile : text; sentinel, startx, starty : integer;
                    var data : carddatatype; var last : boolean);
(* Displays the text from infile until the next file marker sentinel *)

VAR
  c : char;
  i, s, e, m, l, linecount : integer;

BEGIN
  InitCardData (data);
  i := 0;
  read (dfile, c);
  while (ord(c) <> sentinel) and (ord(c) <> ENDFILE) do
    begin
      i := i + 1;
      data[i] := c;
      read (dfile, c)
    end;
  last := ord(c) = ENDFILE;
  s := 1;
  e := 1;
  l := 0;
  linecount := startx;
  gotoxy (startx, starty);
  repeat
    while (data[e] <> ' ') and (data[e] <> chr(3)) do
      begin
        e := e + 1;
        linecount := linecount + 1
      end;

    if linecount > 78
      then
        begin
          l := l + 1;
          gotoxy (startx, starty + (1 * l));
          linecount := startx
        end;

    for m := s to e do
      begin
        if data[m] <> chr(3) then
          if data[m] = chr(2)
            then write (',')
            else write (data[m])
      end;
    if data[e] = chr(3)
      then
        begin
          l := l + 1;
          gotoxy (startx, starty + (1 * l));
          linecount := startx
        end;
    e := e + 1;
    linecount := linecount + 1;
    s := e;
  until e >= i;
  writeln;
END; (*ShowText*)

(* ------------------------------------------------------------------------ *)

PROCEDURE PromptForNext (review, showanswer, next, stop : boolean;
                         var choice : char);
(* Prompts user for what to do next *)

BEGIN
  writeln;
  if review
    then
      begin
        textcolor (magenta);
        write ('R');
        textcolor (14);
        write ('eview, ')
      end;
  if showanswer
    then
      begin
        textcolor (14);
        write ('show ');
        textcolor (magenta);
        write ('A');
        textcolor (14);
        write ('nswer, ')
      end;
  if next
    then
      begin
        textcolor (magenta);
        write ('N');
        textcolor (14);
        write ('ext card, ')
      end;
  if stop
    then
      begin
      textcolor (magenta);
      write ('S');
      textcolor (14);
      write ('top ')
    end;
  write ('? ');
  textcolor (cyan);
  read (kbd, choice);
  while (not (choice in ['R', 'r', 'A', 'a', 'N', 'n', 'S', 's'])) or
        ((choice in ['R', 'r']) and (not (review))) or
        ((choice in ['A', 'a']) and (not (showanswer))) or
        ((choice in ['N', 'n']) and (not (next))) or
        ((choice in ['S', 's']) and (not (stop))) do
    read (kbd, choice);
  writeln (choice);
  textcolor (15);
  writeln
END; (*PromptForNext*)

(* ------------------------------------------------------------------------ *)

PROCEDURE SetupReview (var rfname : fnametype; var rfile : text);
(* Sets up the review file path *)

BEGIN
  textcolor (14);
  write ('Enter filename: ');
  GetString (rfname);
  writeln;
  assign (rfile, rfname);
  rewrite (rfile)
END; (*SetupReview*)

(* ------------------------------------------------------------------------ *)

PROCEDURE MoveToReview (var question, answer : carddatatype;
                        var reviewed : boolean; var rfilename : fnametype;
                        var rfile : text);
(* Moves question & answer to review data file *)

VAR
  l : integer;

BEGIN
  if not reviewed
    then SetupReview (rfilename, rfile);
  reviewed := true;
  for l := 1 to Length (question) do
    write (rfile, question[l]);
  write (rfile, chr(ENDUNIT));
  for l := 1 to Length (answer) do
    write (rfile, answer[l]);
  write (rfile, chr(ENDRECORD))
END; (*MoveToReview*)

(* ------------------------------------------------------------------------ *)

PROCEDURE ReviewFishCards;
(* Manages the actual question/answer display *)

VAR
  rfile, dfile : text;
  reviewed, last : boolean;
  rfilename, fname : fnametype;
  c : char;
  question, answer : carddatatype;

BEGIN
  writeln;
  textcolor (15);
  write ('Please enter file name: ');
  textcolor (cyan);
  GetString (fname);
  assign (dfile, fname);
  reset (dfile);
  reviewed := false;
  DrawCards;
  repeat
    textcolor (15);
    ShowText (dfile, ENDUNIT, 3, 2, question, last);
    gotoxy (1,21);
    if last
      then c := 's'
      else PromptForNext (false, true, false, true, c);
    case c of
      'S', 's' : c := chr(ENDFILE);
      'A', 'a' : begin
                   ShowText (dfile, ENDRECORD, 3, 11, answer, last);
                   gotoxy (1,21);
                   if last
                     then c := 's'
                     else PromptForNext (true, false, true, true, c);
                   case c of
                     'S', 's' : c := chr(ENDFILE);
                     'N', 'n' : c := c;
                     'R', 'r' : MoveToReview (question, answer, reviewed, rfilename, rfile)
                    end
                 end
     end;
    DrawCards
  until ord(c) = ENDFILE;
  if reviewed
    then
      begin
        write (rfile, chr(ENDFILE));
        close (rfile)
      end;
  close (dfile)
END; (*ReviewFishCards*)

(* ------------------------------------------------------------------------- *)

PROCEDURE PromptNewOrAdd;
(* Displays menu for new or add to FishCard file *)

BEGIN
  clrscr;
  textcolor (15);
  writeln ('1. New FishCard file');
  writeln ('2. Add to existing FishCard file');
  writeln ('3. Return to main menu');
  writeln;
  write ('Where do you want to make FishCards? ')
END; (*PromptNewOrAdd*)

(* ------------------------------------------------------------------------- *)

PROCEDURE AskNewOrAdd (var choice : char);
(* Gets the user's choice for new or add *)

BEGIN
  textcolor (cyan);
  read (kbd, choice);
  while not (choice in ['1', '2', '3']) do
    read (kbd, choice);
  writeln (choice)
END; (*AskNewOrAdd*)

(* ------------------------------------------------------------------------- *)

PROCEDURE OpenNew (var dfile : text; var fname : fnametype; var nc : integer);
(* Opens a new FishCard file *)

BEGIN
  writeln;
  textcolor (15);
  write ('Please enter name for file: ');
  textcolor (cyan);
  GetString (fname);
  textcolor (15);
  assign (dfile, fname);
  rewrite (dfile);
  nc := 0
END; (*OpenNew*)

(* ------------------------------------------------------------------------- *)

PROCEDURE CopyFile (var source, dest : text; var nc : integer);
(* Copies file source to dest *)

VAR
  t : char;

BEGIN
  nc := 0;
  while not eof (source) do
    begin
      read (source, t);
      if ord(t) = ENDRECORD
        then nc := nc + 1;
      if t <> chr(ENDFILE)
        then write (dest, t)
    end
END; (*CopyFile*)

(* ------------------------------------------------------------------------- *)

PROCEDURE OpenOld (var dfile : text; var fname : fnametype; var nc : integer);
(* Opens an existing FishCard file *)

VAR
  rfile : text;
  t : char;

BEGIN
  writeln;
  textcolor (15);
  write ('Please enter the file name: ');
  textcolor (cyan);
  GetString (fname);
  assign (dfile, fname);
  reset (dfile);
  assign (rfile, TEMPFILE);
  rewrite (rfile);
  CopyFile (dfile, rfile, nc);
  rewrite (dfile);
  reset (rfile);
  CopyFile (rfile, dfile, nc);
  close (rfile);
  erase (rfile)
END; (*OpenOld*)

(* ------------------------------------------------------------------------- *)

PROCEDURE CreateCards (var dfile : text; var nc : integer);
(* Creates FishCards into the active file *)

VAR
  another : char;
  data : carddatatype;
  i : integer;

BEGIN
  repeat
    DrawCards;
    DrawQAText;
    nc := nc + 1;
    gotoxy (3,2);
    textcolor (cyan);
    GetCardData (data);
    for i := 1 to Length (data) do
      if data[i] = ','
        then write (dfile, chr(2))
        else write (dfile, data[i]);
    write (dfile, chr(ENDUNIT));
    writeln;
    writeln;
    gotoxy (3,11);
    GetCardData (data);
    for i := 1 to Length (data) do
      if data[i] = ','
        then write (dfile, chr(2))
        else write (dfile, data[i]);
    write (dfile, chr(ENDRECORD));
    writeln;
    writeln;
    gotoxy (1,21);
    textcolor (14);
    write ('Another card, Y/N? ');
    read (kbd, another);
    while not (another in ['Y', 'y', 'N', 'n']) do
      read (kbd, another)
  until another in ['n', 'N']
END; (*CreateCards*)

(* ------------------------------------------------------------------------- *)

PROCEDURE CreateNewFishCards;
(* Creates new FishCards *)

VAR
  choice : char;
  dfile : text;
  fname : fnametype;
  numcards : integer;

BEGIN
  PromptNewOrAdd;
  AskNewOrAdd (choice);
  case choice of
      '1' : OpenNew (dfile, fname, numcards);
      '2' : OpenOld (dfile, fname, numcards);
      '3' : choice := choice
    end;
  if choice <> '3'
    then
      begin
        CreateCards (dfile, numcards);
        write (dfile, chr(ENDFILE));
        close (dfile)
      end
END; (*CreateNewFishCards*)

(* ------------------------------------------------------------------------- *)

PROCEDURE ImportExportMenu;
(* Displays menu for import or export *)

BEGIN
  clrscr;
  textcolor (15);
  writeln ('1. Translate file from text to FishCard');
  writeln ('2. Translate file from FishCard to text');
  writeln ('3. Return to main menu');
  writeln;
  write ('Please select an option from the menu above: ');
END; (*ImportExportMenu*)

(* ------------------------------------------------------------------------- *)

PROCEDURE TranslateTextFile (var dfile, rfile : text);
(* Text -> FishCard translation loop *)

VAR
  c : char;

BEGIN
  while not eof (dfile) do
    begin
      while not eoln (dfile) do
        begin
          read (dfile, c);
          if c = ','
            then write (rfile, chr(2))
            else write (rfile, c)
        end;
      readln (dfile);
      write (rfile, chr(ENDUNIT));
      while not eoln (dfile) do
        begin
          read (dfile, c);
          if c = ','
            then write (rfile, chr(2))
            else write (rfile, c)
        end;
      readln (dfile);
      write (rfile, chr(ENDRECORD))
    end;
  write (rfile, chr(ENDFILE))
END; (*TranslateTextFile*)

(* ------------------------------------------------------------------------- *)

PROCEDURE TextToFishCard;
(* Translates a return-delimited text file to FishCard *)

VAR
  tfilename, ffilename : fnametype;
  dfile, rfile : text;
  keep : char;

BEGIN
  textcolor (15);
  write ('Enter TEXT file name: ');
  textcolor (cyan);
  GetString (tfilename);
  writeln;
  textcolor (15);
  write ('Enter name to save as in FishCard format: ');
  textcolor (cyan);
  GetString (ffilename);
  writeln;
  textcolor (15);
  write ('Keep first pair, Y/N? ');
  textcolor (cyan);
  read (kbd, keep);
  while not (keep in ['y', 'Y', 'n', 'N']) do
    read (kbd, keep);
  writeln (keep);
  textcolor (15);
  writeln ('Translating...');
  assign (dfile, tfilename);
  assign (rfile, ffilename);
  reset (dfile);
  rewrite (rfile);
  if keep in ['n', 'N']
    then
      begin
        readln (dfile);
        readln (dfile)
      end;
  TranslateTextFile (dfile, rfile);
  close (dfile);
  close (rfile)
END; (*TextToFishCard*)

(* ------------------------------------------------------------------------- *)

PROCEDURE TranslateFishCardFile (var dfile, rfile : text);
(* FishCard -> Text translation loop *)

VAR
  c : char;
  cnt : integer;

BEGIN
  read (dfile, c);
  while ord(c) <> ENDFILE do
    begin
      while ord(c) <> ENDUNIT do
        begin
          if c = chr(2)
            then write (rfile, ',')
            else write (rfile, c);
          read (dfile, c)
        end;
      read (dfile, c);
      writeln (rfile);
      while ord(c) <> ENDRECORD do
        begin
          if c = chr(2)
            then write (rfile, ',')
            else write (rfile, c);
          read (dfile, c)
        end;
      writeln (rfile);
      read (dfile, c)
    end
END; (*TranslateFishCardFile*)

(* ------------------------------------------------------------------------- *)

PROCEDURE FishCardToText;
(* Translates a FishCard file to a return-delimited text file *)

VAR
  tfilename, ffilename : fnametype;
  dfile, rfile : text;
  title : char;
  q, a : carddatatype;
  index : integer;

BEGIN
  textcolor (15);
  write ('Enter FishCard file name: ');
  textcolor (cyan);
  GetString (ffilename);
  writeln;
  textcolor (15);
  write ('Enter name to save as in TEXT format: ');
  textcolor (cyan);
  GetString (tfilename);
  writeln;
  textcolor (15);
  write ('Add title pair, Y/N? ');
  textcolor (cyan);
  read (kbd, title);
  while not (title in ['y', 'Y', 'n', 'N']) do
    read (kbd, title);
  writeln (title);
  textcolor (15);
  write ('Translating...');
  assign (dfile, ffilename);
  assign (rfile, tfilename);
  reset (dfile);
  rewrite (rfile);
  if title in ['y', 'Y']
    then
      begin
        DrawCards;
        DrawQAText;
        textcolor (cyan);
        gotoxy (3,2);
        GetCardData (q);
        gotoxy (3,11);
        GetCardData (a);
        textcolor (15);
        clrscr;
        writeln ('Translating...');
        for index := 1 to Length (q) do
          write (rfile, q[index]);
        writeln (rfile);
        for index := 1 to Length (a) do
          write (rfile, a[index]);
        writeln (rfile)
      end;
  TranslateFishCardFile (dfile, rfile);
  close (dfile);
  close (rfile)
END; (*FishCardToText*)

(* ------------------------------------------------------------------------- *)

Procedure ImportExport;
(* Import & export cards between text and FishCard format *)

VAR
  c : char;

BEGIN
  ImportExportMenu;
  AskNewOrAdd (c);
  case c of
      '1' : TextToFishCard;
      '2' : FishCardToText;
      '3' : c := c
    end
END; (*ImportExport*)

(* ------------------------------------------------------------------------- *)

PROCEDURE ConfirmQuit (var choice : char);
(* Confirms that the user wants to quit FishCard*)

VAR
  confirm : char;

BEGIN
  textcolor (15);
  writeln;
  write ('Are you sure you want to quit FishCard, Y/N? ');
  read (kbd, confirm);
  while not (confirm in ['Y', 'y', 'N', 'n']) do
    read (kbd, confirm);
  if confirm in ['y', 'Y']
    then clrscr
    else choice := '0'
END; (*ConfirmQuit*)

(* ========================================================================= *)

BEGIN
  repeat
    ShowMenu (choice);
    case choice of
        '0' : choice := choice;
        '1' : CreateNewFishCards;
        '2' : ReviewFishCards;
        '3' : ImportExport;
        '4' : ConfirmQuit (choice)
      end
  until choice = '4'
END.